# Generated by Django 3.2.12 on 2023-05-30 10:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0009_auto_20230522_0720'),
    ]

    operations = [
        migrations.AddField(
            model_name='media',
            name='store_id',
            field=models.TextField(null=True),
        ),
    ]
