# Generated by Django 3.2.12 on 2023-05-03 06:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pgpubsub', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]
