# Generated by Django 3.2.12 on 2023-05-19 11:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_alter_media_id'),
    ]

    operations = [
        migrations.RenameField(
            model_name='media',
            old_name='id',
            new_name='key',
        ),
    ]
