# Generated by Django 3.2.12 on 2022-04-07 10:27

from django.db import migrations, models

try:
    from django.db.models import JSONField
except:
    from django.contrib.postgres.fields import JSONField


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name='ID'
                    ),
                ),
                ('channel', models.CharField(max_length=63)),
                ('payload', JSONField()),
            ],
        ),
    ]
