# Generated by Django 3.2.12 on 2023-05-19 13:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0005_rename_id_media_key'),
    ]

    operations = [
        migrations.CreateModel(
            name='Parent',
            fields=[
                ('key', models.AutoField(primary_key=True, serialize=False)),
            ],
        ),
        migrations.CreateModel(
            name='Child',
            fields=[
                ('parent_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='tests.parent')),
            ],
            bases=('tests.parent',),
        ),
    ]
